clear
pr drop _all
set matsize 800
set more off

***********************
*  Likelihood Evaluator                                                           
***********************

program define splag_ll

args lnf mu rho sigma
tempvar A rSL
gen `A'= ones - `rho'*EIGS1
gen `rSL'=`rho'*SL1
qui replace `lnf'= ln(`A') + ln(normden($ML_y1-`rSL'-`mu', 0, `sigma'))
end

global nobs = 195
global n = 13

***********************
* Open Data For Weights
***********************
clear
use "C:\Documents and Settings\Administrator\Desktop\comparative_political_studies\taxweights.dta", clear
***********************
mkmat var1-var$nobs, matrix(W)
matrix eigenvalues eig1 imaginaryv = W
matrix eig2 = eig1'
matrix ones=J($nobs,1,1)


**************************
* Open Data for Regression
**************************
drop _all
use "C:\Documents and Settings\Administrator\Desktop\comparative_political_studies\taxdata.dta", clear
**************************
global Y  captx
global X  l1captx l1capcon l1trade l1ltunem psdebt l1old l1grow avpcrpro avpcrdin l1leftc l1tcdemc c2-c13 t23 t45 t67 t89 t1011 t1213 t1415
mkmat $Y, matrix(Y)
matrix SL = W*Y
svmat SL, n(SL)
svmat eig2, n(EIGS)
svmat ones, n(ones)

*************************
*Produce starting values 
*************************
qui regress $Y $X
matrix OLSb=e(b)
local OLSsigma=e(rmse)

***************************
*Estimate spatial lag model
***************************

	ml model lf splag_ll (mu: $Y=$X) (rho:) (sigma:), technique(dfp) vce(oim)
	ml init OLSb
	ml init rho:_cons=0
	ml init sigma:_cons=`OLSsigma'
	*ml check
	*ml report
	ml max, difficult ltolerance(1e-8) tolerance(1e-8)

predict yhat
mkmat yhat, matrix(yhat)
matrix eye = I($nobs)
matrix mult = eye - _b[rho:_cons]*W
matrix yhat_m = ((yhat')*inv(mult))'
matrix e2 = (Y-yhat_m)'*(Y-yhat_m)
scalar e2 = e2[1,1]
qui summarize $Y
matrix ym = r(mean)*J($nobs,1,1)
matrix ym2 = (Y-ym)'*(Y-ym)
scalar ym2 = ym2[1,1]
scalar r2 = 1 - (e2/ym2)
scalar list r2


*******************Short-run and Steady-State Effects******************


*************Long-Run Steady-State Spatio-Temporal Effects********************
* Open Data For NxN Weights
***********************
drop _all
use "C:\Documents and Settings\Administrator\Desktop\comparative_political_studies\taxweightsnxn.dta", clear
***********************
mkmat var1-var$n, matrix(W_LR)
matrix eye_n=I($n) 
matrix estcf1_LR = eye_n - _b[rho:_cons]*W_LR - _b[mu:l1captx]*eye_n
matrix estcf_LR = _b[mu:l1ltunem]*inv(estcf1_LR)
matrix d_cf_rho1_LR = estcf_LR*W_LR*inv(estcf1_LR)
matrix d_cf_rho_LR = d_cf_rho1_LR[1...,2] 
matrix d_cf_phi1_LR = estcf_LR*inv(estcf1_LR)
matrix d_cf_phi_LR = d_cf_phi1_LR[1...,2]
matrix d_cf_beta1_LR = inv(estcf1_LR)
matrix d_cf_beta_LR = d_cf_beta1_LR[1...,2]
matrix d_cf_TEMP_LR = [d_cf_rho_LR, d_cf_phi_LR, d_cf_beta_LR]
matrix d_cf_LR = d_cf_TEMP_LR

matrix V = e(V)
matrix VCVM = [V[32,32], V[32,1],V[32,4] \ V[1,32], V[1,1],V[1,4] \V[4,32] ,V[4,1],V[4,4]]
matrix VCVM_2 = [V[32,32], V[32,4] \ V[32,4], V[4,4]]


matrix delta1_LR = d_cf_LR*VCVM*d_cf_LR'
matrix delta_LR = vecdiag(delta1_LR)'
matrix cf_LR = estcf_LR
svmat cf_LR, n(cf_LR)
svmat delta_LR, n(delta_LR)

*************Short-Run Spatial Effects for deind=deind_2********************
matrix estcf1_SR = eye_n - _b[rho:_cons]*W_LR 
matrix estcf_SR = _b[mu:l1ltunem]*inv(estcf1_SR)
matrix d_cf_rho1_SR = estcf_SR*W_LR*inv(estcf1_SR)
matrix d_cf_rho_SR = d_cf_rho1_SR[1...,2] 
matrix d_cf_beta1_SR = inv(estcf1_SR)
matrix d_cf_beta_SR = d_cf_beta1_SR[1...,2]
matrix d_cf_TEMP_SR = [d_cf_rho_SR, d_cf_beta_SR]
matrix d_cf_SR = d_cf_TEMP_SR
matrix delta1_SR = d_cf_SR*VCVM_2*d_cf_SR'
matrix delta_SR = vecdiag(delta1_SR)'
matrix cf_SR = estcf_SR
svmat cf_SR, n(cf_SR)
svmat delta_SR, n(delta_SR)
outfile cf_LR1 delta_LR1 cf_SR1 delta_SR1 using "C:\Documents and Settings\Administrator\Desktop\comparative_political_studies\cf_LR_SR", wide replace



